@@ -17,7 +17,7 @@ To use::
  .. inheritance-diagram:: spatialmath.pose3d
     :top-classes: collections.UserList
     :parts: 1
-    
+
 .. image:: ../figs/pose-values.png
 """
 from __future__ import annotations
@@ -35,6 +35,7 @@ from spatialmath.pose2d import SE2
 from spatialmath.twist import Twist3
 
 from typing import TYPE_CHECKING, Optional
+
 if TYPE_CHECKING:
     from spatialmath.quaternion import UnitQuaternion
 
@@ -341,7 +342,7 @@ class SO3(BasePoseMatrix):
         """
         theta, v = smb.tr2angvec(self.R)
         return theta * v
-    
+
     # ------------------------------------------------------------------------ #
 
     @staticmethod
@@ -455,7 +456,9 @@ class SO3(BasePoseMatrix):
         return cls([smb.rotz(x, unit=unit) for x in smb.getvector(theta)], check=False)
 
     @classmethod
-    def Rand(cls, N: int = 1, *, theta_range:Optional[ArrayLike2] = None, unit: str = "rad") -> Self:
+    def Rand(
+        cls, N: int = 1, *, theta_range: Optional[ArrayLike2] = None, unit: str = "rad"
+    ) -> Self:
         """
         Construct a new SO(3) from random rotation
 
@@ -481,7 +484,13 @@ class SO3(BasePoseMatrix):
 
         :seealso: :func:`spatialmath.quaternion.UnitQuaternion.Rand`
         """
-        return cls([smb.q2r(smb.qrand(theta_range=theta_range, unit=unit)) for _ in range(0, N)], check=False)
+        return cls(
+            [
+                smb.q2r(smb.qrand(theta_range=theta_range, unit=unit))
+                for _ in range(0, N)
+            ],
+            check=False,
+        )
 
     @overload
     @classmethod
@@ -725,6 +734,48 @@ class SO3(BasePoseMatrix):
 
         return cls(np.c_[x, y, z], check=True)
 
+    @classmethod
+    def RotatedVector(cls, v1: ArrayLike3, v2: ArrayLike3, tol=20) -> Self:
+        """
+        Construct a new SO(3) from a vector and its rotated image
+
+        :param v1: initial vector
+        :type v1: array_like(3)
+        :param v2: vector after rotation
+        :type v2: array_like(3)
+        :param tol: tolerance for singularity in units of eps, defaults to 20
+        :type tol: float
+        :return: SO(3) rotation
+        :rtype: :class:`SO3` instance
+
+        ``SO3.RotatedVector(v1, v2)`` is an SO(3) rotation defined in terms of
+        two vectors. The rotation takes vector ``v1`` to ``v2``.
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import SO3
+            >>> v1 = [1, 2, 3]
+            >>> v2 = SO3.Eul(0.3, 0.4, 0.5) * v1
+            >>> print(v2)
+            >>> R = SO3.RotatedVector(v1, v2)
+            >>> print(R)
+            >>> print(R * v1)
+
+        .. note:: The vectors do not have to be unit-length.
+        """
+        # https://math.stackexchange.com/questions/180418/calculate-rotation-matrix-to-align-vector-a-to-vector-b-in-3d
+        v1 = smb.unitvec(v1)
+        v2 = smb.unitvec(v2)
+        v = smb.cross(v1, v2)
+        s = smb.norm(v)
+        if abs(s) < tol * np.finfo(float).eps:
+            return cls(np.eye(3), check=False)
+        else:
+            c = np.dot(v1, v2)
+            V = smb.skew(v)
+            R = np.eye(3) + V + V @ V * (1 - c) / (s**2)
+            return cls(R, check=False)
+
     @classmethod
     def AngleAxis(cls, theta: float, v: ArrayLike3, *, unit: str = "rad") -> Self:
         r"""
@@ -843,22 +894,22 @@ class SO3(BasePoseMatrix):
 
     def UnitQuaternion(self) -> UnitQuaternion:
         """
-            SO3 as a unit quaternion instance
+        SO3 as a unit quaternion instance
 
-            :return: a unit quaternion representation
-            :rtype: UnitQuaternion instance
+        :return: a unit quaternion representation
+        :rtype: UnitQuaternion instance
 
-            ``R.UnitQuaternion()`` is an ``UnitQuaternion`` instance representing the same rotation
-            as the SO3 rotation ``R``.
+        ``R.UnitQuaternion()`` is an ``UnitQuaternion`` instance representing the same rotation
+        as the SO3 rotation ``R``.
 
-            Example:
+        Example:
 
-            .. runblock:: pycon
+        .. runblock:: pycon
 
-                >>> from spatialmath import SO3
-                >>> SO3.Rz(0.3).UnitQuaternion()
+            >>> from spatialmath import SO3
+            >>> SO3.Rz(0.3).UnitQuaternion()
 
-            """
+        """
         # Function level import to avoid circular dependencies
         from spatialmath import UnitQuaternion
 
@@ -931,6 +982,31 @@ class SO3(BasePoseMatrix):
         else:
             return ad
 
+    def mean(self, tol: float = 20) -> SO3:
+        """Mean of a set of SO(3) values
+
+        :param tol: iteration tolerance in units of eps, defaults to 20
+        :type tol: float, optional
+        :return: the mean rotation
+        :rtype: :class:`SO3` instance.
+
+        Computes the Karcher mean of the set of SO(3) rotations within the :class:`SO3` instance.
+
+        :references:
+            - `**Hartley, Trumpf** - "Rotation Averaging" - IJCV 2011 <https://users.cecs.anu.edu.au/~hartley/Papers/PDF/Hartley-Trumpf:Rotation-averaging:IJCV.pdf>`_, Algorithm 1, page 15.
+            - `Karcher mean <https://en.wikipedia.org/wiki/Karcher_mean>`_
+
+        :seealso: :class:`SE3.mean`
+        """
+
+        eta = tol * np.finfo(float).eps
+        R_mean = self[0]  # initial guess
+        while True:
+            r = np.dstack((R_mean.inv() * self).log()).mean(axis=2)
+            if np.linalg.norm(r) < eta:
+                return R_mean
+            R_mean = R_mean @ self.Exp(r)  # update estimate and normalize
+
 
 # ============================== SE3 =====================================#
 
@@ -1246,11 +1322,11 @@ class SE3(SO3):
         """
         if len(self) == 1:
             if order == "zyx":
-                return SE2(self.x, self.y, self.rpy(order = order)[2])
+                return SE2(self.x, self.y, self.rpy(order=order)[2])
             elif order == "xyz":
-                return SE2(self.z, self.y, self.rpy(order = order)[2])
+                return SE2(self.z, self.y, self.rpy(order=order)[2])
             elif order == "yxz":
-                return SE2(self.z, self.x, self.rpy(order = order)[2])
+                return SE2(self.z, self.x, self.rpy(order=order)[2])
         else:
             return SE2([e.yaw_SE2() for e in self])
 
@@ -1292,6 +1368,21 @@ class SE3(SO3):
         else:
             return smb.tr2delta(self.A, X2.A)
 
+    def rtvec(self) -> Tuple[R3, R3]:
+        """
+        Convert to OpenCV-style rotation and translation vectors
+
+        :return: rotation and translation vectors
+        :rtype: ndarray(3), ndarray(3)
+
+        Many OpenCV functions accept pose as two 3-vectors: a rotation vector using
+        exponential coordinates and a translation vector.  This method combines them
+        into an SE(3) instance.
+
+        :seealso: :meth:`rtvec`
+        """
+        return SO3(self).log(twist=True), self.t
+
     def Ad(self) -> R6x6:
         r"""
         Adjoint of SE(3)
@@ -1521,7 +1612,7 @@ class SE3(SO3):
         xrange: Optional[ArrayLike2] = (-1, 1),
         yrange: Optional[ArrayLike2] = (-1, 1),
         zrange: Optional[ArrayLike2] = (-1, 1),
-        theta_range:Optional[ArrayLike2] = None,
+        theta_range: Optional[ArrayLike2] = None,
         unit: str = "rad",
     ) -> SE3:  # pylint: disable=arguments-differ
         """
@@ -1745,7 +1836,7 @@ class SE3(SO3):
         a rotation of ``θ`` about the vector ``v``.
 
         .. math::
-        
+
             \mbox{if}\,\, \theta \left\{ \begin{array}{ll}
                 = 0 & \mbox{return identity matrix}\\
                 \ne 0 & \mbox{v must have a finite length}
@@ -1833,6 +1924,26 @@ class SE3(SO3):
         else:
             return cls(smb.trexp(S), check=False)
 
+    @classmethod
+    def RTvec(cls, rvec: ArrayLike3, tvec: ArrayLike3) -> Self:
+        """
+        Construct a new SE(3) from OpenCV-style rotation and translation vectors
+
+        :param rvec: rotation as exponential coordinates
+        :type rvec: ArrayLike3
+        :param tvec: translation vector
+        :type tvec: ArrayLike3
+        :return: An SE(3) instance
+        :rtype: SE3 instance
+
+        Many OpenCV functions (such as pose estimation) return pose as two 3-vectors: a
+        rotation vector using exponential coordinates and a translation vector.  This
+        method combines them into an SE(3) instance.
+
+        :seealso: :meth:`rtvec`
+        """
+        return SE3.Rt(smb.trexp(rvec), tvec)
+
     @classmethod
     def Delta(cls, d: ArrayLike6) -> SE3:
         r"""
@@ -2000,11 +2111,7 @@ class SE3(SO3):
         return cls(smb.rt2tr(R, t, check=check), check=check)
 
     @classmethod
-    def CopyFrom(
-        cls,
-        T: SE3Array,
-        check: bool = True
-    ) -> SE3:
+    def CopyFrom(cls, T: SE3Array, check: bool = True) -> SE3:
         """
         Create an SE(3) from a 4x4 numpy array that is passed by value.
 
@@ -2089,6 +2196,28 @@ class SE3(SO3):
         else:
             return ad
 
+    def mean(self, tol: float = 20) -> SE3:
+        """Mean of a set of SE(3) values
+
+        :param tol: iteration tolerance in units of eps, defaults to 20
+        :type tol: float, optional
+        :return: the mean SE(3) pose
+        :rtype: :class:`SE3` instance.
+
+        Computes the mean of all the SE(3) values within the :class:`SE3` instance.  Rotations are
+        averaged using the Karcher mean, and translations are averaged using the
+        arithmetic mean.
+
+        :references:
+            - `**Hartley, Trumpf** - "Rotation Averaging" - IJCV 2011 <https://users.cecs.anu.edu.au/~hartley/Papers/PDF/Hartley-Trumpf:Rotation-averaging:IJCV.pdf>`_, Algorithm 1, page 15.
+            - `Karcher mean <https://en.wikipedia.org/wiki/Karcher_mean>`_
+
+        :seealso:  :meth:`SO3.mean`
+        """
+        R_mean = SO3(self).mean(tol)
+        t_mean = self.t.mean(axis=0)
+        return SE3.Rt(R_mean, t_mean)
+
     # @classmethod
     # def SO3(cls, R, t=None, check=True):
     #     if isinstance(R, SO3):
